#include "inputline.hh"

void InputLine::redraw()
{
    if(curspos < xpos)xpos = curspos;
    else if(curspos >= xpos+max)xpos = curspos-max;
    
    region.gotox(promptlen);
    
    BYTE textcolor   = focused ? 0x17 : 0x03;
    BYTE markedcolor = focused ? 0x71 : 0x07;
    
    unsigned left = max;        
    for(unsigned step=0; step<3; ++step)
    {
        // Kolme osaa: Blokkia ennen, blokki ja blokin jlkeen.
        
        unsigned alku, pituus;
        switch(step)
        {
            case 0:
                region.setattr(textcolor);
                alku = xpos;
                if(blockbegin <= alku)
                    pituus = 0;
                else
                    pituus = blockbegin - alku;
                break;
            case 1:
                region.setattr(markedcolor);
                alku = blockbegin; if(alku < xpos)alku = xpos;
                if(blockend <= alku)
                    pituus = 0;
                else
                    pituus = blockend - alku;
                break;
            default:
                region.setattr(textcolor);
                alku = blockend; if(alku < xpos)alku = xpos;
                pituus = result.size() - alku;
                break;
        }
        if(pituus > left)pituus = left;
        region.printf("%s", result.substr(alku, pituus).c_str());
        left -= pituus;
    }
    
    region.setattr(textcolor);        
    region.putchar(focused ? ' ' : '_', left);
    
    if(insertmode)
        Display.UnderlineCursor();
    else
        Display.BlockCursor();
    
    region.gotox(promptlen + curspos-xpos);
    region.flush();
}

bool InputLine::Handle(const InputEvent &ev)
{
    switch(ev.type)
    {
        case inputchar:
            if(!insertmode)result.erase(curspos, 1);
            result.insert(curspos, 1, ev.character);
            ++curspos;
            if(blockbegin > curspos)++blockbegin;
            if(blockend > curspos)++blockend;
            redraw();
            break;
        case inputkey:
            switch(ev.key)
            {
                case KbLt:
                    if(curspos > 0) { --curspos; redraw(); }
                    break;
                case KbRt:
                    if(curspos < result.size()) { ++curspos; redraw(); }
                    break;
                case KbHome:
                    curspos = xpos = 0;
                    redraw();
                    break;
                case KbEnd:
                    curspos = result.size();
                    redraw();
                    break;
                case KbInsert:
                    insertmode = !insertmode;
                    redraw();
                    break;
                case KbBackSpace:
                    if(curspos > 0)--curspos;
                    // Fall thru
                case KbDelete:
                    result.erase(curspos, 1);
                    if(blockbegin > curspos)--blockbegin;
                    if(blockend > curspos)--blockend;
                    redraw();
                    break;
                case CtrlKB:
                    blockbegin = curspos;
                    if(blockend < blockbegin)blockend=blockbegin;
                    redraw();
                    break;
                case CtrlKK:
                    blockend = curspos;
                    if(blockbegin > blockend)blockbegin = blockend;
                    redraw();
                    break;
                case CtrlKC:
                    result.insert(curspos, result.substr(blockbegin, blockend-blockbegin));
                    blockend = blockend + curspos - blockbegin;
                    blockbegin = curspos;
                    redraw();
                    break;
                case CtrlKM:
                case CtrlKV:
                {
                    string tmp = result.substr(blockbegin, blockend-blockbegin);
                    result.erase(blockbegin, tmp.size());
                    if(curspos > blockbegin)curspos -= tmp.size();
                    result.insert(curspos, tmp);
                    blockbegin = curspos;
                    blockend = blockbegin + tmp.size();
                    redraw();
                    break;
                }
                default:
                	return false;
            }
            break;
        case inputctrl:
            switch(ev.character)
            {
                case 'Y':
                    cleartext();
                    redraw();
                    break;
                default:
                	return false;
            }
            break;
        default:
        	return false;
    }
    return true;
}
